/*! \file main.c \brief Test Program Wave Player */
//###########################################################
///	\code #include "../dos.h" \endcode
///	\par Uebersicht
//###################################################################################
//
// Play a wave file from SD-Card
//
// Uses R2R-DAC, or PWM or both;)
//
//#########################################################################
// Last change: 03.10.2010
//#########################################################################
// hk@holger-klabunde.de
// http://www.holger-klabunde.de/index.html
//#########################################################################
// Compiler: AVR-GCC 4.1.1
//#########################################################################
//@{
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>

#include "dos.h"
#include "printf.h"
#include "serial.h"

#define R2R_DAC					0
#define PWM_DAC					1

//#define SAMPLE_RATE				8000
#define SAMPLE_RATE				22050
#define TIMER1_OCR_VALUE		((F_CPU / 8) / SAMPLE_RATE) - 1

// @8000Hz 	and 16MHz F_CPU this is enough for 31ms 8bit mono sounddata
// @22050Hz and 16MHz F_CPU this is enough for 11ms 8bit mono sounddata
#define PLAY_BUFFER_SIZE		256

void PlayWave(char *name);
void StartTimer(void);
void StopTimer(void);

volatile uint8_t play_buffer_index, play_buffer_number;
uint8_t play_buffer0[PLAY_BUFFER_SIZE], play_buffer1[PLAY_BUFFER_SIZE];

//###################################################################################
// Audio output ISR
ISR(TIMER1_COMPA_vect)
//###################################################################################
{

 if(play_buffer_number & 1)
 {
#if R2R_DAC
	PORTA = play_buffer1[play_buffer_index];
#endif
#if PWM_DAC
	OCR2 = play_buffer1[play_buffer_index];
#endif
 }
 else
 {
#if R2R_DAC
	PORTA = play_buffer0[play_buffer_index];
#endif
#if PWM_DAC
	OCR2 = play_buffer0[play_buffer_index];
#endif
 }

 play_buffer_index++;
 if(play_buffer_index == 0) play_buffer_number++;
}

//###################################################################################
/* Hauptprogramm */
int main(void)
//###################################################################################
{

 MMC_IO_Init();

 ser_init();

 sei();              //enable interrupts

 puts("Holgis Wave Player\n");

 if(GetDriveInformation()!=F_OK) // get drive parameters
  {
   puts("No Flash !");
   while(1);
  }
  
 for(;;) // loop forever
  {
   PlayWave("krupps2.wav");
   PlayWave("gypsys8.wav");
  }
}

//###################################################################################
void PlayWave(char *name)
//###################################################################################
{
 unsigned char result;
 unsigned int read_bytes;
 //unsigned long chunksize;
 
 result=Fopen(name,'r');
 if(result==F_OK)
  {
// TODO parse the wave header and use different sample rates or look for the "data" chunk
/*
	read_bytes = Fread(play_buffer0,8); // read "RIFF" chunk + 4 bytes size
	read_bytes = Fread(play_buffer0,4); // read "WAVE" chunk
	read_bytes = Fread(play_buffer0,4); // read "fmt " chunk
	read_bytes = Fread(play_buffer0,4); // read "fmt " chunk 4 bytes size
    chunksize = play_buffer0[0] + play_buffer0[1] << 8UL + play_buffer0[2] << 16UL + play_buffer0[3] << 24UL;
    
	Parse the wave header here
	Skip unknown chunks by simply reading them and then throw away

	read_bytes = Fread(play_buffer0,4); // read "data" chunk
	read_bytes = Fread(play_buffer0,4); // read "data" chunk 4 bytes size
*/
	read_bytes = Fread(play_buffer0,0x3A); // skip the wave header (not always 0x3A length!)

	play_buffer_index = 0; 
	play_buffer_number = 0;

    // fill first audio buffer
	read_bytes = Fread(play_buffer0,PLAY_BUFFER_SIZE);

	StartTimer(); // start playing

	do
	{
	 if(play_buffer_number & 1) // buffer number 1 is playing, read buffer 0
	 {
	   read_bytes = Fread(play_buffer0,PLAY_BUFFER_SIZE);
       while(play_buffer_number & 1); // ugly waiting loop, try to use a state machine
	 }
	 else  // buffer number 0 is playing, read buffer 1
	 {
	   read_bytes = Fread(play_buffer1,PLAY_BUFFER_SIZE);
       while((play_buffer_number & 1) == 0);
	 }

	} while(read_bytes == PLAY_BUFFER_SIZE);
	// if read_bytes is smaller PLAY_BUFFER_SIZE ignore these samples
	// or fill buffer with zero samples at the end
	
	StopTimer();
	Fclose();
  }
}

//###################################################################################
void StartTimer(void)
//###################################################################################
{
 // CTC Mode, Clkio/8 -> 2MHz at 16MHz F_CPU 
 TCCR1A = (0<<COM1A1) | (0<<COM1A0) | (0<<WGM11) | (0<<WGM10);
 TCCR1B = (0<<WGM13) | (1<<WGM12) | (0<<CS12) | (1<<CS11) | (0<<CS10);
 OCR1A = TIMER1_OCR_VALUE;

#if R2R_DAC
 DDRA=0xFF; 	// All outputs
#endif

#if PWM_DAC
 // Fast PWM, no prescaler 
 TCCR2 = (1<<COM21) | (1<<COM20) | (1<<WGM21) | (1<<WGM20) | (1<<CS20);
 DDRD |= (1<<PD7);
#endif

#if defined (__AVR_ATmega644__) || defined (__AVR_ATmega168__)
 TIMSK1 |= (1<<OCIE1A); 
 TIFR1  |= (1<<OCF1A); 
#endif

#if defined (__AVR_ATmega32__)
 TIMSK |= (1<<OCIE1A);
 TIFR  |= (1<<OCF1A); 
#endif
}

//###################################################################################
void StopTimer(void)
//###################################################################################
{
#if defined (__AVR_ATmega644__) || defined (__AVR_ATmega168__)
 TIMSK1 &= ~(1<<OCIE1A); 
#endif

#if defined (__AVR_ATmega32__)
 TIMSK &= ~(1<<OCIE1A);
#endif
}

//@}
